<?php

namespace Teknisa\DF\Modules\Entrada\Services;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DTO\Response;
use Teknisa\DF\Modules\Entrada\Factories\Dfe18500DocumentosParaComposicaoDeCustosFactory;
use Geral\Entities\Itemproduto;


class Dfe18500DocumentosParaComposicaoDeCustosService {
    
    const PROCEDURE_CONHECIMENTO_TRANSPORTE = 'CONHECIMENTO_TRANSPORTE';
    
    protected $entityManager;
    protected $environment;
    protected $documentosComposicaoDeCustosFactory;

    public function __construct(EntityManager $entityManager, $environment, Dfe18500DocumentosParaComposicaoDeCustosFactory $documentosComposicaoDeCustosFactory){
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->documentosComposicaoDeCustosFactory = $documentosComposicaoDeCustosFactory;
    }
    
    public function produtoItemproduto($conditions) {
        $nrlanctonf = $conditions[0]["value"];
        $cdfilial = $conditions[1]["value"];
        $nrorg  =  $conditions[2]["value"];
        
        return $this->documentosComposicaoDeCustosFactory->produtoItemproduto($nrlanctonf, $cdfilial, $nrorg);
    }
    
    public function callProcedureConhecimentoTransporte($cdfilial, $nrlanctonf, $nrlancestq) {
        try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), self::PROCEDURE_CONHECIMENTO_TRANSPORTE);
            $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $nrlanctonf, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCESTQ', Param::PARAM_INPUT, $nrlancestq, Param::PARAM_TYPE_STR));
            $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function calculaValorTotalCustos($cdfilial, $nrlanctonf, $nrorg) {
        $produtos = Itemproduto::findByCdfilialAndNrlanctonfAndNrorg($cdfilial, $nrlanctonf, $nrorg);
        $vrtotalitem =  $this->getValorTotalItem($cdfilial, $nrlanctonf)["VRTOTALITEM"]; 
        $vrrateio = 0;
        $exists_rateioit = count($this->documentosComposicaoDeCustosFactory->getRateioCustoIt($cdfilial, $nrlanctonf)) > 0; 
        
        foreach($produtos as $produto) {
            $produtos = Itemproduto::findByCdfilialAndNrlanctonfAndNrorg($cdfilial, $nrlanctonf, $nrorg);
            $vrtotalcustos = $this->getValorTotalCustos($cdfilial, $nrlanctonf);
            
            if($vrtotalcustos) {
                $vrrateio = ($vrtotalcustos / $vrtotalitem) * $produto->getVrtotaitem();
            }

            if($exists_rateioit) {
                $this->updateRateioit($produto->getCdfilial(), $produto->getNrlanctonf(), $produto->getNrorg(), $produto->getNrseqitprod(), round($vrrateio, 2));
            } else {
                $this->insereRateioit($produto->getCdfilial(), $produto->getNrlanctonf(), $produto->getNrorg(), $produto->getNrseqitprod(), round($vrrateio, 2));
            }
        }
    }
    
    public function insereRateioit($cdfilial, $nrlanctonf, $nrorg, $nrseqitprod, $vrrateio) {
        return $this->documentosComposicaoDeCustosFactory->insereRateioit($cdfilial, $nrlanctonf, $nrorg, $nrseqitprod, $vrrateio);
    }
    
    public function updateRateioit($cdfilial, $nrlanctonf, $nrorg, $nrseqitprod, $vrrateio) {
        return $this->documentosComposicaoDeCustosFactory->updateRateioit($cdfilial, $nrlanctonf, $nrorg, $nrseqitprod, $vrrateio);
    }
    
    public function insereItemNotaFisc($cdfilial, $nrlanctonf, $cdfiliref, $nrlanctonfref) {
        return $this->documentosComposicaoDeCustosFactory->insereItemNotaFisc($cdfilial, $nrlanctonf, $cdfiliref, $nrlanctonfref, $this->environment->getNrOrgTrab());
    }
    
    public function getItemNotaFisc($cdfiliref, $nrlanctonfref, $cdfilial, $nrlanctonf) {
        return $this->documentosComposicaoDeCustosFactory->getItemNotaFisc($cdfiliref, $nrlanctonfref, $cdfilial, $nrlanctonf);
    }
    
    public function getVrRateioCustoItTotal($cdfilial, $nrlanctonf) {
        return $this->documentosComposicaoDeCustosFactory->getVrRateioCustoItTotal($cdfilial, $nrlanctonf);
    }
    
    public function getValorTotalCustos($cdfilial, $nrlanctonf) {
        return $this->documentosComposicaoDeCustosFactory->getValorTotalCustos($cdfilial, $nrlanctonf);
    }
    
    public function getValorTotalItem($cdfilial, $nrlanctonf) {
        return $this->documentosComposicaoDeCustosFactory->getValorTotalItem($cdfilial, $nrlanctonf);
    }
    
    public function insereRelacratcustonf($cdfilial, $nrlanctonf, $cdfilidoccomp, $nrlanctodoccomp) {
        return $this->documentosComposicaoDeCustosFactory->insereRelacratcustonf($cdfilial, $nrlanctonf, $cdfilidoccomp, $nrlanctodoccomp);
    }
    
    
    public function deleteRelacratcustonf($cdfilial, $nrlanctonf, $cdfilidoccomp, $nrlanctodoccomp) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        
        $this->documentosComposicaoDeCustosFactory->deleteRelacratcustonf($cdfilial, $nrlanctonf, $cdfilidoccomp, $nrlanctodoccomp);
    
        $this->entityManager->flush();
        $connection->commit();
    }
    
    public function alteraStatusNotafiscal($cdfilial, $nrlanctonf, $nrorg, $idstatus) {
        return $this->documentosComposicaoDeCustosFactory->alteraStatusNotafiscal($cdfilial, $nrlanctonf, $nrorg, $idstatus);
    }

}