<?php

namespace Teknisa\DF\Modules\Entrada\Services;

use Teknisa\DF\Modules\Entrada\Factories\EntradasOperadorLogisticoFactory;
use Teknisa\DF\Services\LancamentoNotafiscalService;
use Teknisa\Libs\Util\Environment;
use Doctrine\ORM\EntityManager;

use Teknisa\Libs\Util\Masks\PictureProduto;
use Teknisa\Libs\Util\Masks\Masks;

use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

use Estoque\Entities\Lanctoestoq;
use Teknisa\DF\Services\ValidacoesEstoqueService;
    
class EntradasOperadorLogisticoService  {
    
    const NOVO_CODIGO_LANC_ESTOQUE          = 'LANCTOEST';
    const NOVO_CODIGO_NOTA_LANC             = 'LANCNF/FILIAL';
    const PROCEDURE_GERA_PRODUTO            = 'GERA_NOTAFISCAL_PRODUTO';
    const PROCEDURE_GERA_FECHAMENTO_ENTRADA = 'GERA_FECHAMENTO_NFENTRADA';
    const PROCEDURE_CONSULTA_INCIDENCIA_IMP = 'CONSULTA_INCIDENCIA';
    const PROCEDURE_CONSULTA_ALIQUOTA       = 'CONSULTA_ALIQUOTA';
    const PROCEDURE_BAIXA_AF                = 'NF_BAIXA_AF';
    const MSG_ERROS = 
        array("DAT_MAI"          => "Data entrada deve ser menor ou igual a data de processamento",
              "DAT_MEN"          => "Data entrada deve ser maior ou igual a data de processamento.",
              "DAT_M"            => "Data de entrada informada fora do periodo permitido.",
              "LANC_FORTADT"     => "Data de entrada fora do periodo de integração.",
              "LANC_N_GER"       => "Nota Fiscal já lançada.",
              "TOTAL_NF_ZERADO"  => "Existem itens cujo total da NF de saída está zerado.",
              "DTEMISSAO_VAZIA"  => "Nota Fiscal ainda não foi aprovada(sem data de emissão).",
              "DTEMISSAO_MENOR"  => "Nota Fiscal não pode ter data de entrada menor do que a data de emissão de saída.",
              "NRNOTAFISC_VAZIO" => "Nota Fiscal não pode ter data de entrada menor do que a data de emissão de saída.", 
              "QTDE_ZERADA"      => "Existem itens cuja a quantidade ficaria zerada.",
              "PRECO_ZERADO"     => "Existem itens cujo preço ficaria zerado.",
              "PRODUTO_FILIAL"   => "Produto não existe para a filial.",
              "PRODUTO_INATIVO"  => "Produto encontra-se inativo para a filial.",
              "ERRO_ITM"         => "Quantidade de itens da NF de saída diferente da quantidade itens da NF de entrada.",
              "ERRO_TOT"         => "Valor total da NF de saÍda diferente do valor total da NF de entrada.",
              "SERIE_ENTR"       => "Série da nota fiscal de saída não existe para a entrada.");
    
    protected $entradasOperadorLogisticoFactory;
    protected $environment;
    protected $novoCodigo;
    protected $lancamentoNotaService;
    protected $connection;
    protected $masks;
    
    private $prodImpos;
    private $arrayNotasErro    = array();
    private $arrayProdutosErro = array(); 
    private $arrayImpostosErro = array();
    private $notasSucesso      = 0;
    
    public function __construct(EntradasOperadorLogisticoFactory $factory, Environment $environment, EntityManager $entityManager, $novoCodigo, LancamentoNotafiscalService $lancamentoNotaService, ValidacoesEstoqueService $validacoesEstoqueService) {
        $this->connection = $entityManager->getConnection();
        $this->entradasOperadorLogisticoFactory = $factory;
        $this->environment = $environment;
        $this->novoCodigo  = $novoCodigo;
        $this->lancamentoService = $lancamentoNotaService;
        $this->validacoesEstoqueService = $validacoesEstoqueService;
        $this->masks = new Masks();
        date_default_timezone_set('America/Sao_Paulo');
    }   
    
    public function getNotasErro() {
        return $this->arrayNotasErro;
    }
    
    public function gerarEntrada($cdFiliOperLog, $cdFilDest, $cdTipoOper, $dtEntrada, $notasFiscais) {
        
        $paramsGeracao = $this->getDadosGeracao($cdFilDest);
        
        $tipoOperBol = $this->entradasOperadorLogisticoFactory->getTipoOperOpl($cdFiliOperLog, $cdFilDest);
        if(isset($tipoOperBol[0])) $tipoOperEnt = $tipoOperBol[0]["CDTIPOOPERENTR"];
        else $tipoOperEnt = $cdTipoOper;
        
        $organizacao = $this->environment->getNrOrgTrab();
        
        $this->notasSucesso = 0;
        
        foreach($notasFiscais as $notaFiscal) {
            $this->prodImpos = array();
            try {
                $this->connection->beginTransaction();
                // Gera a data de entrada
                $notaFiscal['DTENTRADA'] = ($dtEntrada) ? $dtEntrada : $this->getDataEntrada($cdFiliOperLog, $cdFilDest, $notaFiscal['NRLANCTONF'], $notaFiscal['DTENTRSAID']);
                
                if($this->entradasOperadorLogisticoFactory->verificaNotaSendoUtilizada($cdFilDest, $notaFiscal["NRNOTAFISC"])) {
                    throw new \Exception('Operação bloqueada. A nota fiscal está sendo utilizada por outro operador!');
                }
            
                $feriFeli = $this->entradasOperadorLogisticoFactory->getFeriFeli($notaFiscal['DTENTRADA'], $cdFilDest);
    
                $notaFiscal['DTENTRADA'] = str_replace("/", "-", $notaFiscal["DTENTRADA"]);

                while(!empty($this->entradasOperadorLogisticoFactory->getFeriFeli($notaFiscal['DTENTRADA'], $cdFilDest)[0])) {
                    $notaFiscal['DTENTRADA'] = date('d-m-Y', strtotime("+1 days",strtotime($notaFiscal['DTENTRADA'])));
                }
                
                // Faz as Validações necessarias para nota
                $this->validaNotaEntradaOl($notaFiscal, $paramsGeracao, $cdFilDest, $cdFiliOperLog, $tipoOperEnt, $notaFiscal["DTENTRADA"]);


                $produtos = $this->entradasOperadorLogisticoFactory->buscaProdutosOrigem($cdFiliOperLog, $notaFiscal["NRLANCTONF"], $cdFilDest, $paramsGeracao["CDTIPCUSTEST"], $paramsGeracao["DTPROCESSAFILI"], $paramsGeracao["IDUTORIGPROD"]);
                $tranpNf = $this->entradasOperadorLogisticoFactory->getTranpNf($notaFiscal["CDFILIAL"], $notaFiscal["NRLANCTONF"]);
                
                // Gera novo codigo para nota
                $nrLanctoNfNota = $this->novoCodigo->getCodeByTableName(self::NOVO_CODIGO_NOTA_LANC.$cdFilDest, $organizacao);
                $nrLancEstoq    = $this->novoCodigo->getCodeByTableName(self::NOVO_CODIGO_LANC_ESTOQUE.$cdFilDest, $organizacao);
                 
                // Gera o corpo da nota fiscal de entrada e atualiza com os dados da nota que a gerou
                $proxLancamento = $this->lancamentoService->geraCorpoNotaEntrada($cdFilDest, $cdFilDest, $paramsGeracao["CDEMPRESA"], $tipoOperEnt, $notaFiscal["CDSERIE"], $cdFiliOperLog, 'DFE',
                'S', $nrLanctoNfNota, $tranpNf["IDTPIJURTRAN"], $tranpNf["NRINSJURTRAN"], '', '', $tranpNf["IDPGTOFRETE"],
                '', '', '', '', $nrLancEstoq, $notaFiscal["DTENTRADA"], 0, 0, $notaFiscal["VRNOTAFISC"], '', 0, $notaFiscal["DTEMISSAO"], $notaFiscal["NRNOTAFISC"], '', '');
                
                $nrAcessoNfe = $this->entradasOperadorLogisticoFactory->getNrAcessoNfe($notaFiscal["CDFILIAL"], $notaFiscal["NRLANCTONF"]);
                
                $this->entradasOperadorLogisticoFactory->setNrAcessoNfe($cdFilDest, $nrLanctoNfNota, $nrAcessoNfe, $notaFiscal["CDFILIAL"].$notaFiscal["NRLANCTONF"]);
               
                $sugRegAux   = $this->entradasOperadorLogisticoFactory->getSugRegAux($cdFilDest, $nrLanctoNfNota);
                  
                $geraNota     = true;
                $nrSeqitpeol  = 1;
                $contadorProd = 1;
               
                foreach($produtos as $produto) {
                    
                    try {
                        // Faz as validações necessarias para o produto
                        $this->validaProduto($produto, $notaFiscal, $notaFiscal["CDFILIAL"], $sugRegAux, $cdFilDest);
                        $quantidadeItem = $this->getQuantidadeItemNota($notaFiscal["CDFILIAL"], $notaFiscal["NRLANCTONF"], $produto["CDPRODUTO"], $produto["CDMARCA"], $produto);
                        
                        if(isset($sugRegAux[0])) {
                            $sugRegEntr   = $this->entradasOperadorLogisticoFactory->getSugRegEntr($notaFiscal["CDFILIAL"], $notaFiscal["NRLANCTONF"], $produto["NRSEQITPROD"]);
                            if(!isset($sugRegEntr[0]) || $sugRegAux["CDENCARGOST"]) $valorUniItem = $produto["VRUNITITEM"];
                            else {
                                $valorUniItem = $produto["VRUNITITEM"] + $sugRegEntr["VRIMPOSTO"];
                            }
                        }
                        else {
                            $valorUniItem = $produto["VRUNITITEM"];
                        }
                        
                        $valorItem = $valorUniItem * $quantidadeItem;
                       
                        // Para notas que tem AF
                        if($produto["IDUTILITEM"] == '1') {
                            $nfRelacAf = $this->entradasOperadorLogisticoFactory->getNfRelacAf($cdFilDest, $produto["CDFILLANC"], $proxLancamento["P_PRXLANCTONF"], $produto["NRAUTOFORN"], $produto["CDPRODUTO"], $produto["CDMARCA"], $produto["DTPROGENTAF"]);
                            if(!isset($nfRelacAf[0])) {
                                $this->entradasOperadorLogisticoFactory->insereNfRelacAf($cdFilDest, $produto["CDFILLANC"], $proxLancamento["P_PRXLANCTONF"], $cdFilDest, $produto["NRAUTOFORN"], $produto["CDPRODUTO"], $produto["CDMARCA"], $produto["DTPROGENTAF"], $produto["QTNFRELAFSAI"], $valorUniItem, $organizacao);
                            }
                            $nfRelacAf = $this->entradasOperadorLogisticoFactory->getNfRelacAf($cdFilDest, $produto["CDFILLANC"], $proxLancamento["P_PRXLANCTONF"], $produto["NRAUTOFORN"], $produto["CDPRODUTO"], $produto["CDMARCA"], $produto["DTPROGENTAF"]);
                        
                            $this->entradasOperadorLogisticoFactory->itBaixaAf($produto["CDFILDEST"], $produto["CDFILLANC"], $produto["NRAUTOFORN"], $produto["CDPRODUTO"], $produto["CDMARCA"], $produto["DTPROGENTAF"]);
                            
                            $this->entradasOperadorLogisticoFactory->baixaAf($produto["CDFILDEST"], $produto["CDFILLANC"], $produto["NRAUTOFORN"]);
                        }
                        else {
                            $relacNfOl = $this->entradasOperadorLogisticoFactory->relacNfOl($notaFiscal["CDFILIAL"], $notaFiscal["NRLANCTONF"], $produto["CDPRODUTO"], $produto["CDMARCA"]);
                           
                            foreach ($relacNfOl as $relac) {
                                $relacNfOlProd  =  $this->entradasOperadorLogisticoFactory->relacNfOlProd($cdFilDest, $proxLancamento["P_PRXLANCTONF"], $relac["CDFILDEST"], $relac["CDFILLANC"], $relac["NRPEDEXOL"], $relac["CDMARCA"], $relac["CDPRODUTO"], $relac["DTPROGENPEOL"]);
                                if(!isset($relacNfOlProd[0])) {
                                    $this->entradasOperadorLogisticoFactory->insereNfRelacOlProd($cdFilDest, $relac["CDFILLANC"], $proxLancamento["P_PRXLANCTONF"], $relac["CDFILDEST"], $relac["NRPEDEXOL"], $relac["CDPRODUTO"], $relac["CDMARCA"], $relac["DTPROGENPEOL"], $quantidadeItem, $valorUniItem, $nrSeqitpeol, $organizacao);
                                    $nrSeqitpeol += 1;
                                }
                                else {
                                    $this->entradasOperadorLogisticoFactory->updateNfRelacOlProd($cdFilDest, $relac["CDFILLANC"], $proxLancamento["P_PRXLANCTONF"], $relac["CDFILDEST"], $relac["NRPEDEXOL"], $relac["CDPRODUTO"], $relac["CDMARCA"], $relac["DTPROGENPEOL"]);
                                }
                                
                                $this->entradasOperadorLogisticoFactory->baixaItemOl($relac["CDFILDEST"], $relac["CDFILLANC"], $relac["NRPEDEXOL"], $relac["CDPRODUTO"], $relac["CDMARCA"], $relac["DTPROGENPEOL"]);
                                $this->entradasOperadorLogisticoFactory->baixaPedOl($relac["CDFILDEST"], $relac["CDFILLANC"], $relac["NRPEDEXOL"]);
                            }
                            
                        }
                        
                        if(isset($sugRegEntr[0])) {
                            if(!is_null($sugRegAux[0]["CDENCARGOST"])) $vrICMSST = $sugRegEntr["VRIMPOSTO"] * $quantidadeItem;
                            else $vrICMSST = 0;
                        }
                        else $vrICMSST = 0;
                        // Gera notafiscal por produto
                        $cdProdEmis = $this->maskProduto($produto["CDARVPROD"], $organizacao);
                        
                        $this->lancamentoService->procedureGeraNotaFiscalProduto($cdFilDest, $proxLancamento["P_PRXLANCTONF"], $produto["CDPRODUTO"], $cdProdEmis, $produto["NMPRODUTO"], $quantidadeItem, $valorUniItem, 0, '', '', '', '', '', $produto["IDORIGEMPROD"], 'S', $vrICMSST, $produto["CDMARCA"]);
 
                        $this->criaImpostos($cdFilDest, $proxLancamento["P_PRXLANCTONF"], $tipoOperEnt, $cdFiliOperLog, $notaFiscal["DTENTRADA"], $valorItem, $contadorProd, $produto["IDORIGEMPROD"], $paramsGeracao["CDEMPRESA"],
                                            $produto["CDARVPROD"], $paramsGeracao["CDINSCESTA"], $organizacao);
                                            
                        $seqItProdFormatado = str_pad($contadorProd, 4, "0", STR_PAD_LEFT);
                                    
                        $erro = $this->callBaixaAFNf($cdFilDest, $proxLancamento["P_PRXLANCTONF"], $seqItProdFormatado, 'I', $cdProdEmis, $cdProdEmis, $produto["CDPRODUTO"], $produto["CDPRODUTO"],
                                    $produto["CDMARCA"], $produto["CDMARCA"], $quantidadeItem, $quantidadeItem);
                        
                        if(isset($erro)) if($erro !== 'N') throw new \Exception($erro);
                        
                        $this->entradasOperadorLogisticoFactory->updateItAfRelac($produto["CDFILDEST"], $produto["CDFILLANC"], $produto["NRAUTOFORN"], $produto["CDPRODUTO"], $produto["CDMARCA"], $produto["DTPROGENTAF"]);
                        
                        if($produto["IDUTILITEM"] == '1') {
                            $this->entradasOperadorLogisticoFactory->updateNrAutoFornProd($produto["NRAUTOFORN"], $cdFilDest, $proxLancamento["P_PRXLANCTONF"]);
                        }
                        
                        $contadorProd++;
                    }
                    catch(\Exception $e) {
                        $geraNota = false;
                        array_push($this->arrayProdutosErro, array("CDFILIAL" => $cdFilDest,
                              "NRLANCTONF" => $notaFiscal["NRLANCTONF"],
                              "DTENTRSAID" => $notaFiscal["DTENTRSAID"],
                              "DTENTRADA"  => $notaFiscal["DTENTRADA"],
                              "NMPRODUTO"  => $produto["NMPRODUTO"],
                              "DSERRO"     => $e->getMessage()));
                    }
                }
                
                // Caso não de erro nas operações envolvendo o produto ocorre o fechamento da nota e o commit
                if($geraNota) {
                    $this->procedureGeraFechamentoEntrada($cdFilDest, $proxLancamento["P_PRXLANCTONF"], $proxLancamento["P_PRXLANCESTQ"], $paramsGeracao["DTPROCESSA"], $this->environment->getCdOperador());
                    
                    $lanctoestoq = Lanctoestoq::findOneByCdfilialAndNrlancestqAndNrorg($cdFilDest, $proxLancamento["P_PRXLANCESTQ"], $this->environment->getNrOrgTrab());

                    // NÃO COMENTAR ESTA LINHA EM HIPOTESE ALGUMA
                    // CASO NECESSITE DE ALTERAÇÕES PROCURAR PEDRO MOUTINHO OU RAFAEL DIAS
                    if (is_object($lanctoestoq))
                         $irregularidades = $this->validacoesEstoqueService->verificaIrregularidades($lanctoestoq);


                    $this->geraRateioEntradaOpl($cdFilDest, $proxLancamento["P_PRXLANCTONF"], 'E', '');
                    $paramSupri = $this->entradasOperadorLogisticoFactory->getParamSupri($cdFilDest);
                    if($paramSupri["IDUTCTRLFISOL"] == 'S') {
                        
                        $utControlFisc = $this->utilizaControleFiscal($cdFilDest, 'E', $paramsGeracao);
                        if($utControlFisc) $this->verififcaImpostoObrigatorio($cdFilDest, $proxLancamento["P_PRXLANCTONF"]);
                        
                        $qtItensEntrada = $this->entradasOperadorLogisticoFactory->getQtItemNotaEntrada($cdFilDest, $proxLancamento["P_PRXLANCTONF"]);
                        $qtItensSaida   = $this->entradasOperadorLogisticoFactory->getQtItemNotaSaida($notaFiscal["CDFILIAL"], $notaFiscal["NRLANCTONF"]);
                        if($qtItensEntrada !== $qtItensSaida) throw new \Exception(self::MSG_ERROS["ERRO_ITM"]);
                        
                        $vlrNotaEntrada = $this->entradasOperadorLogisticoFactory->getValorNotaEntrada($cdFilDest, $proxLancamento["P_PRXLANCTONF"]);
                        $vlrNotaSaida   = $this->entradasOperadorLogisticoFactory->getValorNotaSaida($notaFiscal["CDFILIAL"], $notaFiscal["NRLANCTONF"]);
                        
                        if($vlrNotaEntrada !== $vlrNotaSaida) throw new \Exception(self::MSG_ERROS["ERRO_TOT"]);

                    }
                    
                    $this->connection->commit();
                    $this->notasSucesso++;
                    //$this->connection->rollback();
                    
                }
                else $this->connection->rollback();
            }
            catch(\Exception $e) {
                $this->connection->rollback();
                array_push($this->arrayNotasErro, array("CDFILIAL" => $cdFilDest,
                                                  "NRLANCTONF" => $notaFiscal["NRLANCTONF"],
                                                  "DTENTRSAID" => $notaFiscal["DTENTRSAID"],
                                                  "DTENTRADA"  => $notaFiscal["DTENTRADA"],
                                                  "NMPRODUTO"  => '',
                                                  "DSERRO"     => $e->getMessage()));
            }
        }
    }
    
    protected function getDataEntrada($cdFiliOperLog, $cdFilDest, $nrLanctonf, $dtEntrSaid) {
        $dataEmbarqueOl = $this->entradasOperadorLogisticoFactory->getDataEmbarqueOl($cdFiliOperLog, $cdFilDest, $nrLanctonf);
        if($dataEmbarqueOl) {
            if($dataEmbarqueOl[0]) {
                if($dataEmbarqueOl[0]['NRDIAEMBARQUE']) {
                    $nrDiaEmbarque = $dataEmbarqueOl[0]['NRDIAEMBARQUE'];
                    return date('d-m-Y', strtotime("+$nrDiaEmbarque days",strtotime($dtEntrSaid)));
                }
                else return $dtEntrSaid;
            }
            else return $dtEntrSaid;
        }
        else return $dtEntrSaid;
    }
    
    protected function getDadosGeracao($cdFilial) {
        $paramsGeracao = $this->entradasOperadorLogisticoFactory->getParamsGeracao($cdFilial);
        $paramSupri    = $this->entradasOperadorLogisticoFactory->getParamSupri($cdFilial);
        return array_merge($paramsGeracao, $paramSupri);
    }
    
    protected function utilizaControleFiscal($cdFilial, $idEntrSaidOp, $paramsGeracao) {
        if($idEntrSaidOp === 'E') {
            if($paramsGeracao['IDUTCTRLFISENTRPG'] === 'S' || $paramsGeracao['IDUTCTRLFISENTRPF'] === 'S') return true;
            else return true;
        }
        else if($idEntrSaidOp === 'S') {
            if($paramsGeracao['IDUTCTRLFISSAIDPG'] === 'S' || $paramsGeracao['IDUTCTRLFISSAIDPF'] === 'S') return true;
            else return false;
        }
        else return false;
    }
    
    protected function criaImpostos($cdFilial, $nrLanctonf, $cdTipoOper, $cdFiliOperLog, $dataEntrada, $vrBaseCalc, $seqItProd, $origemProd, $cdEmpresa, $cdArvProd, $cdInscEsta, $organizacao) {
       
        $interEstad  = 'N';
        
        $cdGrupoFisc = $this->lancamentoService->callConsultaGrupofiscal('L', $cdFilial, $cdFilial, 'E', $cdTipoOper);
        $endeOrig    = $this->entradasOperadorLogisticoFactory->getEndeFili($cdFiliOperLog)[0];
     
        $endeDest    = $this->entradasOperadorLogisticoFactory->getEndeFili($cdFilial)[0];
        
        if($endeOrig["CDPAIS"] == $endeDest["CDPAIS"] && $endeOrig["SGESTADO"] != $endeDest["SGESTADO"]) {
            $interEstad = 'S';
        }
        

        
        $this->callConsultaIncidenciaImpostos('E', $cdEmpresa, $cdArvProd, $cdInscEsta, $cdTipoOper, $cdFilial, $nrLanctonf, $cdGrupoFisc, $interEstad, $origemProd);
        
        $incidencia = $this->entradasOperadorLogisticoFactory->getIncidencia($cdFilial.$nrLanctonf);
        
        foreach($incidencia as $inc) {
            $existe = false;
            $this->callConsultaAliquota($inc["CDIMPOTEMP"], 'E', $cdEmpresa, $cdArvProd, $cdInscEsta, $endeOrig["CDPAIS"], $endeOrig["SGESTADO"], $endeDest["CDPAIS"],
                $endeDest["SGESTADO"], ($inc["IDUTMUNIORIG"] == 'D') ? $endeDest["CDMUNICIPIO"]: $endeOrig["CDMUNICIPIO"], $vrBaseCalc, $dataEntrada, $cdFilial, $nrLanctonf, $cdGrupoFisc, $cdTipoOper, $origemProd);
    
            $aliquota = $this->entradasOperadorLogisticoFactory->getAliqTemp($cdFilial.$nrLanctonf);
            
            $taxaBc = (100 -  $aliquota["VRPREDBCTMP"])/100;
            $taxaAl = $aliquota["VRALIQUTMP"]  / 100;
            $baseCalc = $vrBaseCalc * $taxaAl;
            $imposto  = ($baseCalc * $taxaAl) - $taxaBc;
            $aliqOl   = $aliquota["VRALIQUTMP"];
            
            $seqItProdFormatado = str_pad($seqItProd, 4, "0", STR_PAD_LEFT);
            
            $this->entradasOperadorLogisticoFactory->insereTrabProImpos($cdFilial, $nrLanctonf, $seqItProdFormatado, $inc["CDIMPOTEMP"], $inc["CDFISCTEMP"], $aliquota["VRPREDBCTMP"],
                $aliquota["VRPARDEDTMP"], $baseCalc, $aliqOl, $imposto, $organizacao);
                
            foreach($this->prodImpos as &$prodImpo) {
                if($prodImpo["CDFILIIMPOIT"] == $cdFilial && $prodImpo["NRLANCIMPOIT"] == $nrLanctonf && $prodImpo["CDIMPOSTOIT"] == $inc["CDIMPOTEMP"]) {
                    $prodImpo["VRIMPOSIT"]      += $imposto;
                    $prodImpo["VRBCALCIMPOIT"]  += $baseCalc;
                    $prodImpo["VRCONTIMPOIT"]   += $baseCalc;
                    $this->entradasOperadorLogisticoFactory->updateTrabImpoNfTot($cdFilial, $nrLanctonf, $inc["CDIMPOTEMP"], $prodImpo["VRIMPOSIT"], '00000', '00000', $prodImpo["VRBCALCIMPOIT"], $organizacao);
                    $existe = true;
                }    
            }
            if(!$existe) {
                $this->entradasOperadorLogisticoFactory->insereTrabImpoNfTot($cdFilial, $nrLanctonf, $inc["CDIMPOTEMP"], $aliqOl, '00000', '00000', $baseCalc, $organizacao);
                $this->prodImpos[] = array("CDFILIIMPOIT" => $cdFilial, "NRLANCIMPOIT" => $nrLanctonf, "NRSEQIMPOIT" => $seqItProdFormatado, "CDIMPOSTOIT" => $inc["CDIMPOTEMP"], "CDFISCIMPOIT" => $inc["CDFISCTEMP"], 
                        "VRREDBCIMPOIT" => $aliquota["VRPREDBCTMP"], "VRPDEDIMPOIT" => $aliquota["VRPARDEDTMP"], "VRCONTIMPOIT" => $baseCalc, "VRBCALCIMPOIT" => $baseCalc, "VRIMPOSIT" => $imposto);
            }
                        
            $this->entradasOperadorLogisticoFactory->deleteAliqTemp($cdFilial.$nrLanctonf);
            $this->entradasOperadorLogisticoFactory->deleteInicTemp($cdFilial.$nrLanctonf);
        }
    }
    
    protected function geraRateioEntradaOpl($cdFilial, $nrLanctonf, $idEntrSaid, $rateioNf, $autMan = 'N', $realizaManut = 'P') {
        if($realizaManut != 'S') {
            if($realizaManut == 'P') {
                $this->entradasOperadorLogisticoFactory->deleteRateioNf($cdFilial, $nrLanctonf);
                $notaFiscal = $this->entradasOperadorLogisticoFactory->getNotaFiscalRateio($cdFilial, $nrLanctonf);
                $consContaRateio = $this->entradasOperadorLogisticoFactory->getConsContaRateio($idEntrSaid, $notaFiscal["CDTIPOOPER"]);
                
                if($idEntrSaid == 'E') {
                    if($consContaRateio["IDCONSCTBRAT"] == 'S') {
                        $contaPagar = $this->entradasOperadorLogisticoFactory->getContaPagar($cdFilial, $nrLanctonf);
                        if(isset($contaPagar[0])) {
                            $tipoConta = $contaPagar[0]["CDTPCTPAGAR"];
                        }
                        else {
                            $tipoConta = $this->entradasOperadorLogisticoFactory->getTpContaNfEntr($cdFilial, $nrLanctonf);
                        }
                        
                        $produtos     = $this->getItemProduto($cdFilial, $nrLanctonf);
                        
                        $rateioCtEntr = $this->getRatTpCtp($tipoConta);
                        
                        foreach ($produtos as $produto) {
                            foreach ($rateioCtEntr as $rat) {
                                $filiRateio = ($rat["CDFILIAL"]) ? $rat["CDFILIAL"] : $notaFiscal["CDFILIRECE"];
                                
                                $rateioProd = $this->getRateioProd($cdFilial, $nrLanctonf, $produto["NRSEQITPROD"], $filiRateio, $rat["CDCENTCUST"]);
                                
                                $vlrRateio  = $produto["VRCONTITEM"] * ($rat["VRPERATTPCTP"] / 100);
                                
                                if($rateioProd[0]) $this->entradasOperadorLogisticoFactory->updateRateioProd($cdFilial, $nrLanctonf, $produto["NRSEQITPROD"], $filiRateio, $vlrRateio, $rat["CDCENTCUST"], $rat["VRPERATTPCTP"]);
                                else $this->entradasOperadorLogisticoFactory->insereRateioProd($cdFilial, $nrLanctonf, $produto["NRSEQITPROD"], $filiRateio, $rat["CDCENTCUST"], $vlrRateio, $rat["VRPERATTPCTP"], $organizacao);
                            }
                        }
                        
                    }
                    else {
                        $this->entradasOperadorLogisticoFactory->insereRateioNfeEntr($cdFilial, $nrLanctonf, $notaFiscal["CDFILIRECE"]);
                    }
                }
            }
            else {
                // Não utilizado por esta rotina
            }
        }
    }
    
    protected function verififcaImpostoObrigatorio($cdFilial, $nrLanctonf) {
        $notaFiscal   = $this->entradasOperadorLogisticoFactory->getDadosNf($cdFilial, $nrLanctonf);
        $impostosObri = $this->entradasOperadorLogisticoFactory->getImpostoObrigatorio($notaFiscal["IDENTRSAIDOP"], $notaFiscal["CDTIPOOPER"]);
        $impostosErro = '';
        $produtosComErro = false;

        foreach ($impostosObri as $impObri) {
            if($impObri["IDPRODNOTA"] == 'P') {
                $impostoProdutos = $this->entradasOperadorLogisticoFactory->getImpostoProdutos($cdFilial, $nrLanctonf, $impObri["CDIMPOSTO"]);

                foreach ($impostoProdutos as $impostoProduto) {
                    $produtosComErro = true;
                    array_push($this->arrayImpostosErro, array("NMPRODUTO"  => $impostoProduto["NMPRODUTO"],
                                                         "DSERRO"     => "Produto sem incidência de imposto obrigatório. Imposto: ".$impObri["SGIMPOSTO"]));
                }
            }
            else {
                $impostosCorpo   = $this->entradasOperadorLogisticoFactory->getImpostoNota($cdFilial, $nrLanctonf, $impObri["CDIMPOSTO"]);
                if(!isset($impostosCorpo[0])) {
                    $notaErro = true;   
                    $impostosErro.= $impObri["SGIMPOSTO"].', ';
                }
                else if($impostosCorpo[0]["CDFILIAL"] == null) {
                    $notaErro = true;   
                    $impostosErro.= $impObri["SGIMPOSTO"].', ';
                }
            }
        }
        
        if($produtosComErro) throw new \Exception("Notafiscal possui produtos sem incidência de imposto(s) obrigatório(s).");
        $size = strlen($impostosErro);

        if($size != 0) {
            $impostosErro = substr($impostosErro,0, $size-2);
            throw new \Exception("Nota Fiscal sem incidência do(s) imposto(s) obrigatório(s) ".$impostosErro.'.');
        }
        
    }
    
    //****************************************PROCEDURES *****************************************//
    
    public function callConsultaIncidenciaImpostos($entrSaidIn, $cdEmpresa, $cdArvProd, $cdInscEsta, $cdTipoOper, $cdFilial, $nrLanctonf, $cdGrupoFisc, $interEstad, $origemProd) {
        
        $procedure = new StoredProcedure($this->connection, self::PROCEDURE_CONSULTA_INCIDENCIA_IMP);
        
        $procedure->addParam(new Param("P_ENTRSAIDIN",      Param::PARAM_INPUT, $entrSaidIn,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_EMPRESA",         Param::PARAM_INPUT, $cdEmpresa,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_PRODUTO",         Param::PARAM_INPUT, $cdArvProd,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_INSCESTA",        Param::PARAM_INPUT, $cdInscEsta,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_TIPOOPER",        Param::PARAM_INPUT, $cdTipoOper,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_FILIAL",          Param::PARAM_INPUT, $cdFilial,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_LANCTONF",        Param::PARAM_INPUT, $nrLanctonf,    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDGRUPOFISC",     Param::PARAM_INPUT, $cdGrupoFisc,   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDGRPFISCSV",     Param::PARAM_INPUT, NULL,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_IDINCINTEREST",   Param::PARAM_INPUT, $interEstad,    Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param("P_IDSISTEMA",       Param::PARAM_INPUT, NULL,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDFILIRECE",      Param::PARAM_INPUT, NULL,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDFORNECED",      Param::PARAM_INPUT, NULL,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDCLIENTE",       Param::PARAM_INPUT, NULL,             Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param("P_INIVALID",        Param::PARAM_INPUT, NULL,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDFILIEMIT",      Param::PARAM_INPUT, NULL,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_VRPRODUTO",       Param::PARAM_INPUT, NULL,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_LANCPRODUTO",     Param::PARAM_INPUT, NULL,             Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param("P_IDORIGEMPROD",    Param::PARAM_INPUT, $origemProd,    Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }
    
    public function callConsultaAliquota($cdImposto, $entrSaidAl, $cdEmpresa, $cdProduto, $cdInscEsta, $cdPaisOrig, $cdEstadoOrig, $cdPaisDest, 
        $cdEstadoDest, $cdMuniOrig, $vrBaseCalc, $iniInvalid, $cdFilial, $nrLanctonf, $cdGrupoFisc, $cdTipoOper, $origemProd) {
         
        $procedure = new StoredProcedure($this->connection, self::PROCEDURE_CONSULTA_ALIQUOTA);
 
        $procedure->addParam(new Param("P_IMPOSTO",           Param::PARAM_INPUT, $cdImposto,       Param::PARAM_TYPE_STR));   
        $procedure->addParam(new Param("P_ENTRSAIDAL",        Param::PARAM_INPUT, $entrSaidAl,      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_EMPRESA",           Param::PARAM_INPUT, $cdEmpresa,       Param::PARAM_TYPE_STR));   
        $procedure->addParam(new Param("P_PRODUTO",           Param::PARAM_INPUT, $cdProduto,       Param::PARAM_TYPE_STR));   
        $procedure->addParam(new Param("P_INSCESTA",          Param::PARAM_INPUT, $cdInscEsta,      Param::PARAM_TYPE_STR));  
        $procedure->addParam(new Param("P_PAISORIG",          Param::PARAM_INPUT, $cdPaisOrig,      Param::PARAM_TYPE_STR));  
        $procedure->addParam(new Param("P_ESTAORIG",          Param::PARAM_INPUT, $cdEstadoOrig,    Param::PARAM_TYPE_STR));  
        $procedure->addParam(new Param("P_PAISDEST",          Param::PARAM_INPUT, $cdPaisDest,      Param::PARAM_TYPE_STR));  
        $procedure->addParam(new Param("P_ESTADEST",          Param::PARAM_INPUT, $cdEstadoDest,    Param::PARAM_TYPE_STR));  
        $procedure->addParam(new Param("P_MUNIORIG",          Param::PARAM_INPUT, $cdMuniOrig,      Param::PARAM_TYPE_STR));  
        $procedure->addParam(new Param("P_BASECALC",          Param::PARAM_INPUT, $vrBaseCalc,      Param::PARAM_TYPE_STR));  
        $procedure->addParam(new Param("P_INIVALID",          Param::PARAM_INPUT, $iniInvalid,      Param::PARAM_TYPE_STR));  
        $procedure->addParam(new Param("P_FILIAL",            Param::PARAM_INPUT, $cdFilial,        Param::PARAM_TYPE_STR));    
        $procedure->addParam(new Param("P_LANCTONF",          Param::PARAM_INPUT, $nrLanctonf,      Param::PARAM_TYPE_STR));  
        $procedure->addParam(new Param("P_CDGRUPOFISC",       Param::PARAM_INPUT, $cdGrupoFisc,     Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDTIPOOPER",        Param::PARAM_INPUT, $cdTipoOper,      Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param("P_IDSISTEMA",           Param::PARAM_INPUT, null,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_LANCPRODUTO",         Param::PARAM_INPUT, null,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_VRPRODUTO",           Param::PARAM_INPUT, null,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDFILIRECE",          Param::PARAM_INPUT, null,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDFORNECED",          Param::PARAM_INPUT, null,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_CDCLIENTE",           Param::PARAM_INPUT, null,             Param::PARAM_TYPE_STR));
        
        $procedure->addParam(new Param("P_IDORIGEMPROD",      Param::PARAM_INPUT, $origemProd,      Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }
    
    public function callBaixaAFNf($cdFilial, $nrLanctonf, $nrSeqItProd, $idStatus, $newCdProdEmis, $oldCdProdEmis, $newCdProduto, $oldCdProduto,
                $newCdMarca, $oldCdMarca, $newQtItProd, $oldQtItProd) {
        $procedure = new StoredProcedure($this->connection, self::PROCEDURE_BAIXA_AF);
        
        $procedure->addParam(new Param("P_CDFILIAL",          Param::PARAM_INPUT,  $cdFilial,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_NRLANCTONF",        Param::PARAM_INPUT,  $nrLanctonf,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_NRSEQITPROD",       Param::PARAM_INPUT,  $nrSeqItProd,            Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_IDSTATUS",          Param::PARAM_INPUT,  $idStatus,               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_NEWCDPRODEMIS",     Param::PARAM_INPUT,  $newCdProdEmis,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_OLDCDPRODEMIS",     Param::PARAM_INPUT,  $oldCdProdEmis,          Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_NEWCDPRODUTO",      Param::PARAM_INPUT,  $newCdProduto,           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_OLDCDPRODUTO",      Param::PARAM_INPUT,  $oldCdProduto,           Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_NEWCDMARCA",        Param::PARAM_INPUT,  $newCdMarca,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_OLDCDMARCA",        Param::PARAM_INPUT,  $oldCdMarca,             Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_NEWQTITPROD",       Param::PARAM_INPUT,  $newQtItProd,            Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_OLDQTITPROD",       Param::PARAM_INPUT,  $oldQtItProd,            Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param("P_IDERRO",            Param::PARAM_OUTPUT, '',                      Param::PARAM_TYPE_STR, 10));
        
        $output = $procedure->execute();

        return $output['P_IDERRO'];
    }
    
    private function procedureGeraFechamentoEntrada($filial, $numLancamento, $numLancEstoque, $dtProcessamento, $operador = '') {
        $procedure = new StoredProcedure($this->connection, self::PROCEDURE_GERA_FECHAMENTO_ENTRADA);
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $filial, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $numLancamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DATAPROC', Param::PARAM_INPUT, $dtProcessamento, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRLANCESTQ', Param::PARAM_INPUT, $numLancEstoque, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $operador, Param::PARAM_TYPE_STR));
        $procedure->execute();
    }
    
    /********************************** VALIDACOES DA NOTAFISCAL **********************************
    * Faz as validações necessarias para a nota                                                   *
    * Caso a nota não seja aprovada em alguma das validações uma exceção é retornada              *
    * e a nota é adicionada ao array de notas com erro $arrayNotasErro                            *
    ***********************************************************************************************/
    protected function validaNotaEntradaOl($notaFiscal, $paramsGeracao, $cdFilDest, $cdFiliOperLog, $tipoOperEnt, $dtEntrada) {
        $this->validacoesParamCtrlFisOl($notaFiscal['DTENTRADA'], $paramsGeracao, $paramsGeracao["DTPROCESSA"], 
                                        $paramsGeracao["NRDIASENTRFIL"], $paramsGeracao["NRDIASENTR"], 
                                        $paramsGeracao["NRDIASENTRPD"], $paramsGeracao["NRDIASENTRPDFIL"]);  
        $this->validaSerieExistente($notaFiscal["CDSERIE"]);
        $this->validaDataIntegracao($paramsGeracao["CDEMPRESA"], 'E', $tipoOperEnt, $notaFiscal["DTENTRADA"]);
        $this->validaNotaFiscalExistente($cdFilDest, $cdFiliOperLog, $notaFiscal["NRNOTAFISC"]);
        $this->validaValorNota($notaFiscal["VRNOTAFISC"]);
        $this->validaDataEmissao($notaFiscal["DTEMISSAO"], $dtEntrada);
        $this->validaNrNotaFisc($notaFiscal["NRNOTAFISC"]);
    }
    
    private function validacoesParamCtrlFisOl($dtEntrada, $paramsGeracao, $dataProcessamento, $nrDiasEntrFil, $nrDiasEntr, $nrDiasEntrPd, $nrDiasEntrPdFil) {
        
        $dataProcessamento = $this->formataDataParaComparacao($dataProcessamento);
        if($paramsGeracao["IDLANCDTMAI"] === 'N' 
            && $paramsGeracao["IDLANCDTMAIFIL"] === 'N'
            && (strtotime($dtEntrada) > strtotime($dataProcessamento))) 
                        throw new \Exception(self::MSG_ERROS["DAT_MAI"]);
         
        if($paramsGeracao["IDLANCDTMEN"] === 'N' 
            && $paramsGeracao["IDLANCDTMENFIL"] === 'N' 
            && (strtotime($dtEntrada) < strtotime($dataProcessamento))) 
                        throw new \Exception(self::MSG_ERROS["DAT_MEN"]); 
            
            
            
        if($paramsGeracao["IDLANCDTMENFIL"] === 'S' && (strtotime($dtEntrada) < strtotime($dataProcessamento))) {
            if(strtotime($dtEntrada) < strtotime("-$nrDiasEntrFil days", strtotime($dataProcessamento)))  
                        throw new \Exception(self::MSG_ERROS["DAT_M"]); 
        }
        else {
            if($paramsGeracao["IDLANCDTMEN"] === 'S' && strtotime($dtEntrada) < strtotime($dataProcessamento)) 
                if(strtotime($dtEntrada) < strtotime("-$nrDiasEntr days", strtotime($dataProcessamento)))
                        throw new \Exception(self::MSG_ERROS["DAT_M"]);
        }
        
        if($paramsGeracao["IDLANCDTMAIFIL"] === 'S' && strtotime($dtEntrada) > strtotime($dataProcessamento)) {
            if(strtotime($dtEntrada) > strtotime("+$nrDiasEntrPdFil days", strtotime($dataProcessamento))) 
                    throw new \Exception(self::MSG_ERROS["DAT_M"]);
        } 
        else {
            if($paramsGeracao["IDLANCDTMAI"] === 'S' && strtotime($dtEntrada) > strtotime($dataProcessamento)) 
                if(strtotime($dtEntrada) > strtotime("+$nrDiasEntrPd days", strtotime($dataProcessamento)))
                        throw new \Exception(self::MSG_ERROS["DAT_M"]);
        }
    }
    
    private function validaSerieExistente($cdSerie) {
        $serie = $this->entradasOperadorLogisticoFactory->getSerieEntr($cdSerie);
        if(empty($serie)) throw new \Exception(self::MSG_ERROS["SERIE_ENTR"]);
    }
    
    private function validaDataIntegracao($cdEmpresa, $idEntrSaidOp, $cdTipoOperac, $dtReferencia) {
      
        $contopctbl = $this->entradasOperadorLogisticoFactory->getContopectbl($cdEmpresa);
        if($contopctbl) {
            if($contopctbl[0]) {
                if($idEntrSaidOp == 'E') {
                    if (strtotime($dtReferencia) >= $this->getDataTime($contopctbl[0]["DTINIINTCONTDFE"])
                        && strtotime($dtReferencia) <= $this->getDataTime($contopctbl[0]["DTFININTCONTDFE"])) 
                            return true;
                    else throw new \Exception(self::MSG_ERROS["LANC_FORTADT"]);
                }
                    
                else if($idEntrSaidOp == 'S') {
                
                    if (strtotime($dtReferencia) >= strtotime($contopctbl[0]["DTINIINTCONTDFS"])
                        && strtotime($dtReferencia) <= strtotime($contopctbl[0]["DTFININTCONTDFS"])) 
                            return true;
                    else throw new \Exception(self::MSG_ERROS["LANC_FORTADT"]);
                }
            }      
            else return true;
        }
        else return true;  
            
    }
    
    private function validaNotaFiscalExistente($cdFilDest, $cdFiliOperLog, $nrNotaFisc) {
        $notaFiscal = $this->entradasOperadorLogisticoFactory->getNotaFiscal($cdFilDest, $cdFiliOperLog, $nrNotaFisc);  
        if(isset($notaFiscal[0])) throw new \Exception(self::MSG_ERROS["LANC_N_GER"]);
    }
    
    private function validaValorNota($vrNotaFisc) {
        if(!($vrNotaFisc > 0)) throw new \Exception(self::MSG_ERROS["TOTAL_NF_ZERADO"]);
    }
    
    private function validaDataEmissao($dtEmissao, $dtEntrada) {
        if(is_null($dtEmissao)) throw new \Exception(self::MSG_ERROS["DTEMISSAO_VAZIA"]);
        else if($this->getDataTime($dtEmissao) > $this->getDataTime($dtEntrada)) throw new \Exception(self::MSG_ERROS["DTEMISSAO_MENOR"]); 
    }
    
    private function validaNrNotaFisc($nrNotaFisc) {
        if(is_null($nrNotaFisc)) throw new \Exception(self::MSG_ERROS["NRNOTAFISC_VAZIO"]);
    }
    
    /**************************** VALIDACOES DOS PRODUTOS DA NOTAFISCAL ****************************
    * Faz as validações necessarias para o produto                                                 *
    * Caso o produto não seja aprovada em alguma das validações uma exceção é retornada            *
    * e o produto é adicionada ao array de produtos com erro $arrayNotasErro                       *
    ************************************************************************************************/
    protected function validaProduto($produto, $notaFiscal, $cdFilial, $sugRegAux, $cdFilDest) {
        $this->validaProdutoPorFilial($cdFilDest, $produto["CDPRODUTO"]);
        $this->validaValorProduto($produto["VRUNITITEM"]);
        $this->validaQuantidadeProd($cdFilial, $notaFiscal["NRLANCTONF"], $produto["CDPRODUTO"], $produto["CDMARCA"], $produto);
    }
    
    private function validaProdutoPorFilial($cdFilDest, $cdProduto) {
        $prodFili = $this->entradasOperadorLogisticoFactory->getProdFili($cdFilDest, $cdProduto); 
        if(isset($prodFili[0])) {
            if($prodFili[0]['IDPRODATIV'] !== 'A') throw new \Exception(self::MSG_ERROS["PRODUTO_INATIVO"]); 
        }
        else throw new \Exception(self::MSG_ERROS["PRODUTO_FILIAL"]);
    }
    
    private function validaValorProduto($valorProduto) {
        if(!($valorProduto > 0)) throw new \Exception(self::MSG_ERROS["PRECO_ZERADO"]);
    }
    
    private function validaQuantidadeProd($cdFilial, $nrLanctonf, $cdProduto, $cdMarca, $produto) {
        $quantidadeItem = $this->getQuantidadeItemNota($cdFilial, $nrLanctonf, $cdProduto, $cdMarca, $produto);
        if(!isset($quantidadeItem)) throw new \Exception(self::MSG_ERROS["QTDE_ZERADA"]);
    }
    
    /************************************************************************************************/
    
    protected function getDataTime($data) {
        $data = $this->formataDataParaComparacao($data);
        return strtotime($data);
    }
    
    protected function formataDataParaComparacao($data) {
        $data = substr($data, 0, 10);
        return str_replace("/", "-", $data);
    }
    
    private function maskProduto($cdarvprod, $nrorg) {
        return $this->masks->mask($cdarvprod, preg_replace("/([^.])/", '#', PictureProduto::mascaraProduto($nrorg)));
    }
    
    private function getQuantidadeItemNota($cdFilial, $nrLanctonf, $cdProduto, $cdMarca, $produto) {
        $qtItemNota = $this->entradasOperadorLogisticoFactory->getQtItemNota($cdFilial, $nrLanctonf, $cdProduto, $cdMarca);
        $quantidadeItem = '';                   
        if(count($qtItemNota) !== 0){
            if(count($qtItemNota) == 1) $quantidadeItem = $qtItemNota[0]["QTITEMPROD"];
            else{
                foreach ($qtItemNota as $qtProd) {
                    if($qtProd == $produto["QTNFRELAFSAI"]) $quantidadeItem = $qtProd;
                }
            }
        } 
        else {
            $quantidadeItem = $produto["QTNFRELAFSAI"];
        } 
        return $quantidadeItem;
    }
    
    /************************************************************************************************/
    
    public function getErros() {
        return array_merge($this->getArrayNotasErro(), $this->getArrayProdutosErro(), $this->getArrayImpostosErro());
    }
    
    public function getQtNotasSucesso() {
        return $this->notasSucesso;
    }
    
    public function getArrayNotasErro() {
        return $this->arrayNotasErro;
    } 
    
    public function getArrayProdutosErro() {
        return $this->arrayProdutosErro;
    }
    
    public function getArrayImpostosErro() {
        return $this->arrayImpostosErro;
    }
    
}
